==> create-fly-1.21.11-pre1-6.0.8-1-sources.jar: com/zurrtum/create/infrastructure/transfer/FluidInventoryStorage.java <==
package com.zurrtum.create.infrastructure.transfer;

import com.google.common.collect.MapMaker;
import com.zurrtum.create.infrastructure.fluids.FluidInventory;
import com.zurrtum.create.infrastructure.fluids.FluidStack;
import net.fabricmc.fabric.api.transfer.v1.fluid.FluidVariant;
import net.fabricmc.fabric.api.transfer.v1.storage.SlottedStorage;
import net.fabricmc.fabric.api.transfer.v1.storage.base.SingleSlotStorage;
import net.fabricmc.fabric.impl.transfer.DebugMessages;
import net.minecraft.class_2350;
import net.minecraft.class_2586;
import net.minecraft.class_9326;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.UnmodifiableView;

import java.util.List;
import java.util.Map;
import java.util.Objects;

public interface FluidInventoryStorage extends SlottedStorage<FluidVariant> {
    Map<FluidVariant, FluidStack> CACHE = new MapMaker().weakKeys().makeMap();

    static FluidStack getCachedStack(FluidVariant variant) {
        return CACHE.computeIfAbsent(variant, FluidInventoryStorage::toStack);
    }

    static FluidStack toStack(FluidVariant variant) {
        return new FluidStack(variant.getFluid(), 1, variant.getComponents());
    }

    static FluidInventoryStorage of(FluidInventory inventory) {
        Objects.requireNonNull(inventory, "Null fluid inventory is not supported.");
        return FluidInventoryStorageImpl.of(inventory);
    }

    static FluidInventoryStorage of(FluidInventory inventory, @Nullable class_2350 direction) {
        Objects.requireNonNull(inventory, "Null fluid inventory is not supported.");
        return FluidInventoryStorageImpl.of(inventory, direction);
    }

    static boolean matches(FluidVariant variant, FluidStack stack) {
        if (!variant.isOf(stack.getFluid())) {
            return false;
        }
        class_9326 stackComponents = stack.getComponentChanges();
        class_9326 variantComponents = variant.getComponents();
        if (stackComponents == variantComponents) {
            return true;
        }
        return stackComponents.field_49591.reference2ObjectEntrySet().containsAll(variantComponents.field_49591.reference2ObjectEntrySet());
    }

    @Override
    @UnmodifiableView
    List<SingleSlotStorage<FluidVariant>> getSlots();

    @Override
    default int getSlotCount() {
        return getSlots().size();
    }

    @Override
    default SingleSlotStorage<FluidVariant> getSlot(int slot) {
        return getSlots().get(slot);
    }

    @SuppressWarnings("UnstableApiUsage")
    static String toString(FluidInventory inventory) {
        if (inventory == null) {
            return "~~NULL~~";
        } else {
            String result = inventory.toString();

            if (inventory instanceof class_2586 blockEntity) {
                result += " (%s, %s)".formatted(
                    blockEntity.method_11010(),
                    DebugMessages.forGlobalPos(blockEntity.method_10997(), blockEntity.method_11016())
                );
            }

            return result;
        }
    }
}

==> create-fly-6.0.7-1.21.8-19-sources.jar: com/zurrtum/create/infrastructure/transfer/FluidInventoryStorage.java <==
package com.zurrtum.create.infrastructure.transfer;

import com.google.common.collect.MapMaker;
import com.zurrtum.create.infrastructure.fluids.FluidInventory;
import com.zurrtum.create.infrastructure.fluids.FluidStack;
import net.fabricmc.fabric.api.transfer.v1.fluid.FluidVariant;
import net.fabricmc.fabric.api.transfer.v1.storage.SlottedStorage;
import net.fabricmc.fabric.api.transfer.v1.storage.base.SingleSlotStorage;
import net.fabricmc.fabric.impl.transfer.DebugMessages;
import net.minecraft.class_2350;
import net.minecraft.class_2586;
import net.minecraft.class_9326;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.UnmodifiableView;

import java.util.List;
import java.util.Map;
import java.util.Objects;

public interface FluidInventoryStorage extends SlottedStorage<FluidVariant> {
    Map<FluidVariant, FluidStack> CACHE = new MapMaker().weakKeys().makeMap();

    static FluidStack getCachedStack(FluidVariant variant) {
        return CACHE.computeIfAbsent(variant, FluidInventoryStorage::toStack);
    }

    static FluidStack toStack(FluidVariant variant) {
        return new FluidStack(variant.getFluid(), 1, variant.getComponents());
    }

    static FluidInventoryStorage of(FluidInventory inventory) {
        Objects.requireNonNull(inventory, "Null fluid inventory is not supported.");
        return FluidInventoryStorageImpl.of(inventory);
    }

    static FluidInventoryStorage of(FluidInventory inventory, @Nullable class_2350 direction) {
        Objects.requireNonNull(inventory, "Null fluid inventory is not supported.");
        return FluidInventoryStorageImpl.of(inventory, direction);
    }

    static boolean matches(FluidVariant variant, FluidStack stack) {
        if (!variant.isOf(stack.getFluid())) {
            return false;
        }
        class_9326 stackComponents = stack.getComponentChanges();
        class_9326 variantComponents = variant.getComponents();
        if (stackComponents == variantComponents) {
            return true;
        }
        return stackComponents.field_49591.reference2ObjectEntrySet()
            .containsAll(variantComponents.field_49591.reference2ObjectEntrySet());
    }

    @Override
    @UnmodifiableView
    List<SingleSlotStorage<FluidVariant>> getSlots();

    @Override
    default int getSlotCount() {
        return getSlots().size();
    }

    @Override
    default SingleSlotStorage<FluidVariant> getSlot(int slot) {
        return getSlots().get(slot);
    }

    @SuppressWarnings("UnstableApiUsage")
    static String toString(FluidInventory inventory) {
        if (inventory == null) {
            return "~~NULL~~";
        } else {
            String result = inventory.toString();

            if (inventory instanceof class_2586 blockEntity) {
                result += " (%s, %s)".formatted(
                    blockEntity.method_11010(),
                    DebugMessages.forGlobalPos(blockEntity.method_10997(), blockEntity.method_11016())
                );
            }

            return result;
        }
    }
}
